@@ -50,6 +50,7 @@
 #include <utils/qtcassert.h>
 
 #include <QDebug>
+#include <QElapsedTimer>
 #include <QFileInfo>
 #include <QLoggingCategory>
 #include <QSharedPointer>
@@ -57,10 +58,11 @@
 #include <QThread>
 
 using namespace ClangBackEnd;
-using namespace ClangCodeModel;
-using namespace ClangCodeModel::Internal;
 using namespace ProjectExplorer;
 
+namespace ClangCodeModel {
+namespace Internal {
+
 static Q_LOGGING_CATEGORY(debug, "qtc.clangcodemodel.batch", QtWarningMsg);
 
 static int timeOutFromEnvironmentVariable()
@@ -77,7 +79,7 @@ static int timeOutFromEnvironmentVariable()
     return intervalAsInt;
 }
 
-static int timeOutInMs()
+int timeOutInMs()
 {
     static int timeOut = timeOutFromEnvironmentVariable();
     return timeOut;
@@ -234,9 +236,9 @@ bool OpenProjectCommand::run()
     QTC_ASSERT(openProjectSucceeded, return false);
 
     Project *project = openProjectSucceeded.project();
-    project->configureAsExampleProject({});
+    project->configureAsExampleProject(nullptr);
 
-    return CppTools::Tests::TestCase::waitUntilCppModelManagerIsAwareOf(project, timeOutInMs());
+    return CppTools::Tests::TestCase::waitUntilProjectIsFullyOpened(project, timeOutInMs());
 }
 
 Command::Ptr OpenProjectCommand::parse(BatchFileLineTokenizer &arguments,
@@ -302,7 +304,7 @@ WaitForUpdatedCodeWarnings::WaitForUpdatedCodeWarnings(ClangEditorDocumentProces
 
 bool WaitForUpdatedCodeWarnings::wait(int timeOutInMs) const
 {
-    QTime time;
+    QElapsedTimer time;
     time.start();
 
     forever {
@@ -468,7 +470,7 @@ bool CompleteCommand::run()
     auto *processor = ClangEditorDocumentProcessor::get(documentFilePath);
     QTC_ASSERT(processor, return false);
 
-    return completionResults(editor, QStringList(), timeOutInMs());
+    return !completionResults(editor, QStringList(), timeOutInMs()).isNull();
 }
 
 Command::Ptr CompleteCommand::parse(BatchFileLineTokenizer &arguments,
@@ -580,7 +582,7 @@ bool ProcessEventsCommand::run()
 {
     qCDebug(debug) << "line" << context().lineNumber << "ProcessEventsCommand" << m_durationInMs;
 
-    QTime time;
+    QElapsedTimer time;
     time.start();
 
     forever {
@@ -746,9 +748,6 @@ bool BatchFileParser::parseLine(const QString &line)
 
 } // anonymous namespace
 
-namespace ClangCodeModel {
-namespace Internal {
-
 static QString applySubstitutions(const QString &filePath, const QString &text)
 {
     const QString dirPath = QFileInfo(filePath).absolutePath();
