@@ -44,6 +44,8 @@ SearchResultTreeView::SearchResultTreeView(QWidget *parent)
     setIndentation(14);
     setUniformRowHeights(true);
     setExpandsOnDoubleClick(true);
+    header()->setSectionResizeMode(QHeaderView::ResizeToContents);
+    header()->setStretchLastSection(false);
     header()->hide();
 
     connect(this, &SearchResultTreeView::activated,
@@ -55,12 +57,12 @@ void SearchResultTreeView::setAutoExpandResults(bool expand)
     m_autoExpandResults = expand;
 }
 
-void SearchResultTreeView::setTextEditorFont(const QFont &font, const SearchResultColor &color)
+void SearchResultTreeView::setTextEditorFont(const QFont &font, const SearchResultColors &colors)
 {
-    m_model->setTextEditorFont(font, color);
+    m_model->setTextEditorFont(font, colors);
 
     QPalette p;
-    p.setColor(QPalette::Base, color.textBackground);
+    p.setColor(QPalette::Base, colors.value(SearchResultColor::Style::Default).textBackground);
     setPalette(p);
 }
 
@@ -78,6 +80,28 @@ void SearchResultTreeView::addResults(const QList<SearchResultItem> &items, Sear
     }
 }
 
+void SearchResultTreeView::keyPressEvent(QKeyEvent *event)
+{
+    if ((event->key() == Qt::Key_Return
+            || event->key() == Qt::Key_Enter)
+            && event->modifiers() == 0
+            && currentIndex().isValid()
+            && state() != QAbstractItemView::EditingState) {
+        const SearchResultItem item
+            = model()->data(currentIndex(), ItemDataRoles::ResultItemRole).value<SearchResultItem>();
+        emit jumpToSearchResult(item);
+        return;
+    }
+    TreeView::keyPressEvent(event);
+}
+
+bool SearchResultTreeView::event(QEvent *e)
+{
+    if (e->type() == QEvent::Resize)
+        header()->setMinimumSectionSize(width());
+    return TreeView::event(e);
+}
+
 void SearchResultTreeView::emitJumpToSearchResult(const QModelIndex &index)
 {
     if (model()->data(index, ItemDataRoles::IsGeneratedRole).toBool())
@@ -89,7 +113,7 @@ void SearchResultTreeView::emitJumpToSearchResult(const QModelIndex &index)
 
 void SearchResultTreeView::setTabWidth(int tabWidth)
 {
-    SearchResultTreeItemDelegate *delegate = static_cast<SearchResultTreeItemDelegate *>(itemDelegate());
+    auto delegate = static_cast<SearchResultTreeItemDelegate *>(itemDelegate());
     delegate->setTabWidth(tabWidth);
     doItemsLayout();
 }
