@@ -41,9 +41,12 @@
 #include "nodelistproperty.h"
 #include "nodeproperty.h"
 #include <rewriterview.h>
+#include "annotation.h"
+
+#include <utils/algorithm.h>
 
 #include <QHash>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QSet>
 #include <QTextStream>
 
@@ -90,7 +93,25 @@ ModelNode::ModelNode(const ModelNode &modelNode, AbstractView *view)
       m_model(modelNode.model()),
       m_view(view)
 {
+}
 
+ModelNode::ModelNode(ModelNode &&other)
+    : m_internalNode(std::move(other.m_internalNode))
+    , m_model(std::move(other.m_model))
+    , m_view(std::move(other.m_view))
+{
+    other.m_model = {};
+    other.m_view = {};
+}
+
+ModelNode &ModelNode::operator=(ModelNode &&other)
+{
+    ModelNode newNode;
+
+    swap(other, newNode);
+    swap(*this, newNode);
+
+    return *this;
 }
 
 /*! \brief contructs a invalid model node
@@ -100,30 +121,15 @@ ModelNode::ModelNode(const ModelNode &modelNode, AbstractView *view)
 ModelNode::ModelNode():
         m_internalNode(new InternalNode)
 {
-
 }
 
-ModelNode::ModelNode(const ModelNode &other):
-        m_internalNode(other.m_internalNode),
-        m_model(other.m_model),
-        m_view(other.m_view)
-{
-}
+ModelNode::ModelNode(const ModelNode &other) = default;
 
-ModelNode& ModelNode::operator=(const ModelNode &other)
-{
-    this->m_model = other.m_model;
-    this->m_internalNode = other.m_internalNode;
-    this->m_view = other.m_view;
-
-    return *this;
-}
+ModelNode& ModelNode::operator=(const ModelNode &other) = default;
 
 /*! \brief does nothing
 */
-ModelNode::~ModelNode()
-{
-}
+ModelNode::~ModelNode() = default;
 
 /*! \brief returns the name of node which is a short cut to a property like objectName
 \return name of the node
@@ -210,7 +216,9 @@ static bool isIdToAvoid(const QString& id)
         "layer",
         "scale",
         "enabled",
-        "anchors"
+        "anchors",
+        "texture",
+        "shaderInfo"
     };
 
     return ids.contains(id);
@@ -218,8 +226,8 @@ static bool isIdToAvoid(const QString& id)
 
 static bool idContainsWrongLetter(const QString& id)
 {
-    static QRegExp idExpr(QStringLiteral("[a-z_][a-zA-Z0-9_]*"));
-    return !idExpr.exactMatch(id);
+    static QRegularExpression idExpr(QStringLiteral("^[a-z_][a-zA-Z0-9_]+$"));
+    return !id.contains(idExpr);
 }
 
 bool ModelNode::isValidId(const QString &id)
@@ -656,6 +664,16 @@ QList<BindingProperty> ModelNode::bindingProperties() const
     return propertyList;
 }
 
+QList<SignalHandlerProperty> ModelNode::signalProperties() const
+{
+    QList<SignalHandlerProperty> propertyList;
+
+    foreach (const AbstractProperty &property, properties())
+        if (property.isSignalHandlerProperty())
+            propertyList.append(property.toSignalHandlerProperty());
+    return propertyList;
+}
+
 /*!
 \brief removes a property from this node
 \param name name of the property
@@ -675,35 +693,33 @@ void ModelNode::removeProperty(const PropertyName &name) const
         model()->d->removeProperty(internalNode()->property(name));
 }
 
-
 /*! \brief removes this node from the node tree
 */
-
-static QList<ModelNode> descendantNodes(const ModelNode &parent)
+static QList<ModelNode> descendantNodes(const ModelNode &node)
 {
-    QList<ModelNode> descendants(parent.directSubModelNodes());
-    foreach (const ModelNode &child, parent.directSubModelNodes()) {
+    const QList<ModelNode> children = node.directSubModelNodes();
+    QList<ModelNode> descendants = children;
+    for (const ModelNode &child : children)
         descendants += descendantNodes(child);
-    }
+
     return descendants;
 }
 
 static void removeModelNodeFromSelection(const ModelNode &node)
 {
-    { // remove nodes from the active selection:
-        QList<ModelNode> selectedList = node.view()->selectedModelNodes();
+    // remove nodes from the active selection
+    QList<ModelNode> selectedList = node.view()->selectedModelNodes();
 
-        foreach (const ModelNode &childModelNode, descendantNodes(node))
-            selectedList.removeAll(childModelNode);
-        selectedList.removeAll(node);
+    const QList<ModelNode> descendants = descendantNodes(node);
+    for (const ModelNode &descendantNode : descendants)
+        selectedList.removeAll(descendantNode);
 
-        node.view()->setSelectedModelNodes(selectedList);
-    }
+    selectedList.removeAll(node);
+
+    node.view()->setSelectedModelNodes(selectedList);
 }
 
-
 /*! \brief complete removes this ModelNode from the Model
-
 */
 void ModelNode::destroy()
 {
@@ -789,6 +805,19 @@ const QList<ModelNode> ModelNode::directSubModelNodes() const
     return toModelNodeList(internalNode()->allDirectSubNodes(), view());
 }
 
+const QList<ModelNode> ModelNode::directSubModelNodesOfType(const TypeName &typeName) const
+{
+    return Utils::filtered(directSubModelNodes(), [typeName](const ModelNode &node){
+        return node.metaInfo().isValid() && node.metaInfo().isSubclassOf(typeName);
+    });
+}
+
+const QList<ModelNode> ModelNode::subModelNodesOfType(const TypeName &typeName) const
+{
+    return Utils::filtered(allSubModelNodes(), [typeName](const ModelNode &node){
+        return node.metaInfo().isValid() && node.metaInfo().isSubclassOf(typeName);
+    });
+}
 
 /*!
 \brief returns all ModelNodes that are direct or indirect children of this ModelNode
@@ -932,6 +961,9 @@ bool ModelNode::hasNodeListProperty(const PropertyName &name) const
 
 static bool recursiveAncestor(const ModelNode &possibleAncestor, const ModelNode &node)
 {
+    if (!node.isValid())
+        return false;
+
     if (node.hasParentProperty()) {
         if (node.parentProperty().parentModelNode() == possibleAncestor)
            return true;
@@ -1005,7 +1037,7 @@ QVariant ModelNode::toVariant() const
     return QVariant::fromValue(*this);
 }
 
-QVariant ModelNode::auxiliaryData(const PropertyName &name) const
+const QVariant ModelNode::auxiliaryData(const PropertyName &name) const
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -1019,7 +1051,7 @@ void ModelNode::setAuxiliaryData(const PropertyName &name, const QVariant &data)
     m_model.data()->d->setAuxiliaryData(internalNode(), name, data);
 }
 
-void ModelNode::removeAuxiliaryData(const PropertyName &name)
+void ModelNode::removeAuxiliaryData(const PropertyName &name) const
 {
     Internal::WriteLocker locker(m_model.data());
     m_model.data()->d->removeAuxiliaryData(internalNode(), name);
@@ -1041,6 +1073,201 @@ QHash<PropertyName, QVariant> ModelNode::auxiliaryData() const
     return internalNode()->auxiliaryData();
 }
 
+QString ModelNode::customId() const
+{
+    QString result;
+    if (hasCustomId())
+        result = auxiliaryData(customIdProperty).value<QString>();
+
+    return result;
+}
+
+bool ModelNode::hasCustomId() const
+{
+    return hasAuxiliaryData(customIdProperty);
+}
+
+void ModelNode::setCustomId(const QString &str)
+{
+    setAuxiliaryData(customIdProperty, QVariant::fromValue<QString>(str));
+}
+
+void ModelNode::removeCustomId()
+{
+    if (hasCustomId()) {
+        removeAuxiliaryData(customIdProperty);
+    }
+}
+
+QVector<Comment> ModelNode::comments() const
+{
+    return annotation().comments();
+}
+
+bool ModelNode::hasComments() const
+{
+    return annotation().hasComments();
+}
+
+void ModelNode::setComments(const QVector<Comment> &coms)
+{
+    Annotation anno = annotation();
+    anno.setComments(coms);
+
+    setAnnotation(anno);
+}
+
+void ModelNode::addComment(const Comment &com)
+{
+    Annotation anno = annotation();
+    anno.addComment(com);
+
+    setAnnotation(anno);
+}
+
+bool ModelNode::updateComment(const Comment &com, int position)
+{
+    bool result = false;
+    if (hasAnnotation()) {
+        Annotation anno = annotation();
+
+        if (anno.updateComment(com, position)) {
+            setAnnotation(anno);
+            result = true;
+        }
+    }
+
+    return result;
+}
+
+Annotation ModelNode::annotation() const
+{
+    Annotation result;
+
+    if (hasAnnotation())
+        result.fromQString(auxiliaryData(annotationProperty).value<QString>());
+
+    return result;
+}
+
+bool ModelNode::hasAnnotation() const
+{
+    return hasAuxiliaryData(annotationProperty);
+}
+
+void ModelNode::setAnnotation(const Annotation &annotation)
+{
+    setAuxiliaryData(annotationProperty, QVariant::fromValue<QString>(annotation.toQString()));
+}
+
+void ModelNode::removeAnnotation()
+{
+    if (hasAnnotation()) {
+        removeAuxiliaryData(annotationProperty);
+    }
+}
+
+Annotation ModelNode::globalAnnotation() const
+{
+    Annotation result;
+    ModelNode root = view()->rootModelNode();
+
+    if (hasGlobalAnnotation())
+        result.fromQString(root.auxiliaryData(globalAnnotationProperty).value<QString>());
+
+    return result;
+}
+
+bool ModelNode::hasGlobalAnnotation() const
+{
+    return view()->rootModelNode().hasAuxiliaryData(globalAnnotationProperty);
+}
+
+void ModelNode::setGlobalAnnotation(const Annotation &annotation)
+{
+    view()->rootModelNode().setAuxiliaryData(globalAnnotationProperty,
+                                             QVariant::fromValue<QString>(annotation.toQString()));
+}
+
+void ModelNode::removeGlobalAnnotation()
+{
+    if (hasGlobalAnnotation()) {
+        view()->rootModelNode().removeAuxiliaryData(globalAnnotationProperty);
+    }
+}
+
+GlobalAnnotationStatus ModelNode::globalStatus() const
+{
+    GlobalAnnotationStatus result;
+    ModelNode root = view()->rootModelNode();
+
+    if (hasGlobalAnnotation()) {
+        result.fromQString(root.auxiliaryData(globalAnnotationStatus).value<QString>());
+    }
+
+    return result;
+}
+
+bool ModelNode::hasGlobalStatus() const
+{
+    return view()->rootModelNode().hasAuxiliaryData(globalAnnotationStatus);
+}
+
+void ModelNode::setGlobalStatus(const GlobalAnnotationStatus &status)
+{
+    view()->rootModelNode().setAuxiliaryData(globalAnnotationStatus,
+                                             QVariant::fromValue<QString>(status.toQString()));
+}
+
+void ModelNode::removeGlobalStatus()
+{
+    if (hasGlobalStatus()) {
+        view()->rootModelNode().removeAuxiliaryData(globalAnnotationStatus);
+    }
+}
+
+bool ModelNode::locked() const
+{
+    if (hasLocked())
+        return auxiliaryData(lockedProperty).toBool();
+
+    return false;
+}
+
+bool ModelNode::hasLocked() const
+{
+    return hasAuxiliaryData(lockedProperty);
+}
+
+void ModelNode::setLocked(bool value)
+{
+    if (value) {
+        setAuxiliaryData(lockedProperty, true);
+        // Remove newly locked node and all its descendants from potential selection
+        for (ModelNode node : allSubModelNodesAndThisNode()) {
+            node.deselectNode();
+            node.removeAuxiliaryData("timeline_expanded");
+            node.removeAuxiliaryData("transition_expanded");
+        }
+    } else {
+        removeAuxiliaryData(lockedProperty);
+    }
+}
+
+bool ModelNode::isThisOrAncestorLocked(const ModelNode &node)
+{
+    if (!node.isValid())
+        return false;
+
+    if (node.locked())
+        return true;
+
+    if (node.isRootNode() || !node.hasParentProperty())
+        return false;
+
+    return isThisOrAncestorLocked(node.parentProperty().parentModelNode());
+}
+
 void  ModelNode::setScriptFunctions(const QStringList &scriptFunctionList)
 {
     model()->d->setScriptFunctions(internalNode(), scriptFunctionList);
